"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AgentConfig = void 0;
const constants_1 = require("../constants");
const error_1 = require("../error");
const logger_1 = require("../logger");
const CredentialAutoAcceptType_1 = require("../modules/credentials/models/CredentialAutoAcceptType");
const ProofAutoAcceptType_1 = require("../modules/proofs/models/ProofAutoAcceptType");
const types_1 = require("../types");
class AgentConfig {
    constructor(initConfig, agentDependencies) {
        var _a;
        this.initConfig = initConfig;
        this.label = initConfig.label;
        this.logger = (_a = initConfig.logger) !== null && _a !== void 0 ? _a : new logger_1.ConsoleLogger(logger_1.LogLevel.off);
        this.agentDependencies = agentDependencies;
        const { mediatorConnectionsInvite, clearDefaultMediator, defaultMediatorId } = this.initConfig;
        const allowOne = [mediatorConnectionsInvite, clearDefaultMediator, defaultMediatorId].filter((e) => e !== undefined);
        if (allowOne.length > 1) {
            throw new error_1.AriesFrameworkError(`Only one of 'mediatorConnectionsInvite', 'clearDefaultMediator' and 'defaultMediatorId' can be set as they negate each other`);
        }
    }
    /**
     * @deprecated use connectToIndyLedgersOnStartup from the `LedgerModuleConfig` class
     */
    get connectToIndyLedgersOnStartup() {
        var _a;
        return (_a = this.initConfig.connectToIndyLedgersOnStartup) !== null && _a !== void 0 ? _a : true;
    }
    /**
     * @todo remove once did registrar module is available
     */
    get publicDidSeed() {
        return this.initConfig.publicDidSeed;
    }
    /**
     * @deprecated use indyLedgers from the `LedgerModuleConfig` class
     */
    get indyLedgers() {
        var _a;
        return (_a = this.initConfig.indyLedgers) !== null && _a !== void 0 ? _a : [];
    }
    /**
     * @todo move to context configuration
     */
    get walletConfig() {
        return this.initConfig.walletConfig;
    }
    /**
     * @deprecated use autoAcceptConnections from the `ConnectionsModuleConfig` class
     */
    get autoAcceptConnections() {
        var _a;
        return (_a = this.initConfig.autoAcceptConnections) !== null && _a !== void 0 ? _a : false;
    }
    /**
     * @deprecated use autoAcceptProofs from the `ProofsModuleConfig` class
     */
    get autoAcceptProofs() {
        var _a;
        return (_a = this.initConfig.autoAcceptProofs) !== null && _a !== void 0 ? _a : ProofAutoAcceptType_1.AutoAcceptProof.Never;
    }
    /**
     * @deprecated use autoAcceptCredentials from the `CredentialsModuleConfig` class
     */
    get autoAcceptCredentials() {
        var _a;
        return (_a = this.initConfig.autoAcceptCredentials) !== null && _a !== void 0 ? _a : CredentialAutoAcceptType_1.AutoAcceptCredential.Never;
    }
    get didCommMimeType() {
        var _a;
        return (_a = this.initConfig.didCommMimeType) !== null && _a !== void 0 ? _a : types_1.DidCommMimeType.V0;
    }
    /**
     * @deprecated use mediatorPollingInterval from the `RecipientModuleConfig` class
     */
    get mediatorPollingInterval() {
        var _a;
        return (_a = this.initConfig.mediatorPollingInterval) !== null && _a !== void 0 ? _a : 5000;
    }
    /**
     * @deprecated use mediatorPickupStrategy from the `RecipientModuleConfig` class
     */
    get mediatorPickupStrategy() {
        return this.initConfig.mediatorPickupStrategy;
    }
    /**
     * @deprecated use maximumMessagePickup from the `RecipientModuleConfig` class
     */
    get maximumMessagePickup() {
        var _a;
        return (_a = this.initConfig.maximumMessagePickup) !== null && _a !== void 0 ? _a : 10;
    }
    /**
     * @deprecated use baseMediatorReconnectionIntervalMs from the `RecipientModuleConfig` class
     */
    get baseMediatorReconnectionIntervalMs() {
        var _a;
        return (_a = this.initConfig.baseMediatorReconnectionIntervalMs) !== null && _a !== void 0 ? _a : 100;
    }
    /**
     * @deprecated use maximumMediatorReconnectionIntervalMs from the `RecipientModuleConfig` class
     */
    get maximumMediatorReconnectionIntervalMs() {
        var _a;
        return (_a = this.initConfig.maximumMediatorReconnectionIntervalMs) !== null && _a !== void 0 ? _a : Number.POSITIVE_INFINITY;
    }
    /**
     * Encode keys in did:key format instead of 'naked' keys, as stated in Aries RFC 0360.
     *
     * This setting will not be taken into account if the other party has previously used naked keys
     * in a given protocol (i.e. it does not support Aries RFC 0360).
     */
    get useDidKeyInProtocols() {
        var _a;
        return (_a = this.initConfig.useDidKeyInProtocols) !== null && _a !== void 0 ? _a : true;
    }
    get endpoints() {
        // if endpoints is not set, return queue endpoint
        // https://github.com/hyperledger/aries-rfcs/issues/405#issuecomment-582612875
        if (!this.initConfig.endpoints || this.initConfig.endpoints.length === 0) {
            return [constants_1.DID_COMM_TRANSPORT_QUEUE];
        }
        return this.initConfig.endpoints;
    }
    /**
     * @deprecated use mediatorInvitationUrl from the `RecipientModuleConfig` class
     */
    get mediatorConnectionsInvite() {
        return this.initConfig.mediatorConnectionsInvite;
    }
    /**
     * @deprecated use autoAcceptMediationRequests from the `MediatorModuleConfig` class
     */
    get autoAcceptMediationRequests() {
        var _a;
        return (_a = this.initConfig.autoAcceptMediationRequests) !== null && _a !== void 0 ? _a : false;
    }
    /**
     * @deprecated you can use `RecipientApi.setDefaultMediator` to set the default mediator.
     */
    get defaultMediatorId() {
        return this.initConfig.defaultMediatorId;
    }
    /**
     * @deprecated you can set the `default` tag to `false` (or remove it completely) to clear the default mediator.
     */
    get clearDefaultMediator() {
        var _a;
        return (_a = this.initConfig.clearDefaultMediator) !== null && _a !== void 0 ? _a : false;
    }
    get useLegacyDidSovPrefix() {
        var _a;
        return (_a = this.initConfig.useLegacyDidSovPrefix) !== null && _a !== void 0 ? _a : false;
    }
    /**
     * @todo move to context configuration
     */
    get connectionImageUrl() {
        return this.initConfig.connectionImageUrl;
    }
    get autoUpdateStorageOnStartup() {
        var _a;
        return (_a = this.initConfig.autoUpdateStorageOnStartup) !== null && _a !== void 0 ? _a : false;
    }
    extend(config) {
        return new AgentConfig(Object.assign(Object.assign(Object.assign({}, this.initConfig), { logger: this.logger, label: this.label }), config), this.agentDependencies);
    }
    toJSON() {
        var _a, _b, _c, _d;
        return Object.assign(Object.assign({}, this.initConfig), { walletConfig: Object.assign(Object.assign({}, this.walletConfig), { key: ((_a = this.walletConfig) === null || _a === void 0 ? void 0 : _a.key) ? '[*****]' : undefined, storage: Object.assign(Object.assign({}, (_b = this.walletConfig) === null || _b === void 0 ? void 0 : _b.storage), { credentials: ((_d = (_c = this.walletConfig) === null || _c === void 0 ? void 0 : _c.storage) === null || _d === void 0 ? void 0 : _d.credentials) ? '[*****]' : undefined }) }), logger: this.logger.logLevel, agentDependencies: Boolean(this.agentDependencies), label: this.label });
    }
}
exports.AgentConfig = AgentConfig;
//# sourceMappingURL=AgentConfig.js.map