"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CredentialsModule = void 0;
const models_1 = require("../../agent/models");
const CredentialsApi_1 = require("./CredentialsApi");
const CredentialsModuleConfig_1 = require("./CredentialsModuleConfig");
const indy_1 = require("./formats/indy");
const JsonLdCredentialFormatService_1 = require("./formats/jsonld/JsonLdCredentialFormatService");
const services_1 = require("./protocol/revocation-notification/services");
const v1_1 = require("./protocol/v1");
const v2_1 = require("./protocol/v2");
const repository_1 = require("./repository");
class CredentialsModule {
    constructor(config) {
        this.api = CredentialsApi_1.CredentialsApi;
        this.config = new CredentialsModuleConfig_1.CredentialsModuleConfig(config);
    }
    /**
     * Registers the dependencies of the credentials module on the dependency manager.
     */
    register(dependencyManager, featureRegistry) {
        // Api
        dependencyManager.registerContextScoped(CredentialsApi_1.CredentialsApi);
        // Config
        dependencyManager.registerInstance(CredentialsModuleConfig_1.CredentialsModuleConfig, this.config);
        // Services
        dependencyManager.registerSingleton(v1_1.V1CredentialService);
        dependencyManager.registerSingleton(services_1.RevocationNotificationService);
        dependencyManager.registerSingleton(v2_1.V2CredentialService);
        // Repositories
        dependencyManager.registerSingleton(repository_1.CredentialRepository);
        // Features
        featureRegistry.register(new models_1.Protocol({
            id: 'https://didcomm.org/issue-credential/1.0',
            roles: ['holder', 'issuer'],
        }), new models_1.Protocol({
            id: 'https://didcomm.org/issue-credential/2.0',
            roles: ['holder', 'issuer'],
        }), new models_1.Protocol({
            id: 'https://didcomm.org/revocation_notification/1.0',
            roles: ['holder'],
        }), new models_1.Protocol({
            id: 'https://didcomm.org/revocation_notification/2.0',
            roles: ['holder'],
        }));
        // Credential Formats
        dependencyManager.registerSingleton(indy_1.IndyCredentialFormatService);
        dependencyManager.registerSingleton(JsonLdCredentialFormatService_1.JsonLdCredentialFormatService);
    }
}
exports.CredentialsModule = CredentialsModule;
//# sourceMappingURL=CredentialsModule.js.map