"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.V2IssueCredentialHandler = void 0;
const models_1 = require("../../../../../agent/models");
const V2IssueCredentialMessage_1 = require("../messages/V2IssueCredentialMessage");
const V2RequestCredentialMessage_1 = require("../messages/V2RequestCredentialMessage");
class V2IssueCredentialHandler {
    constructor(credentialService, didCommMessageRepository, logger) {
        this.supportedMessages = [V2IssueCredentialMessage_1.V2IssueCredentialMessage];
        this.credentialService = credentialService;
        this.didCommMessageRepository = didCommMessageRepository;
        this.logger = logger;
    }
    async handle(messageContext) {
        const credentialRecord = await this.credentialService.processCredential(messageContext);
        const shouldAutoRespond = await this.credentialService.shouldAutoRespondToCredential(messageContext.agentContext, {
            credentialRecord,
            credentialMessage: messageContext.message,
        });
        if (shouldAutoRespond) {
            return await this.acceptCredential(credentialRecord, messageContext);
        }
    }
    async acceptCredential(credentialRecord, messageContext) {
        this.logger.info(`Automatically sending acknowledgement with autoAccept`);
        const requestMessage = await this.didCommMessageRepository.findAgentMessage(messageContext.agentContext, {
            associatedRecordId: credentialRecord.id,
            messageClass: V2RequestCredentialMessage_1.V2RequestCredentialMessage,
        });
        const { message } = await this.credentialService.acceptCredential(messageContext.agentContext, {
            credentialRecord,
        });
        if (messageContext.connection) {
            return new models_1.OutboundMessageContext(message, {
                agentContext: messageContext.agentContext,
                connection: messageContext.connection,
                associatedRecord: credentialRecord,
            });
        }
        else if ((requestMessage === null || requestMessage === void 0 ? void 0 : requestMessage.service) && messageContext.message.service) {
            const recipientService = messageContext.message.service;
            const ourService = requestMessage.service;
            return new models_1.OutboundMessageContext(message, {
                agentContext: messageContext.agentContext,
                serviceParams: {
                    service: recipientService.resolvedDidCommService,
                    senderKey: ourService.resolvedDidCommService.recipientKeys[0],
                },
            });
        }
        this.logger.error(`Could not automatically create credential ack`);
    }
}
exports.V2IssueCredentialHandler = V2IssueCredentialHandler;
//# sourceMappingURL=V2IssueCredentialHandler.js.map